#
# This will take annotations and then plug them into a page.
#
# 08/25/00 Ben Tyszka
# 11/02/00 Gerd Kortemeyer
# 11/07/00 Ben Tyszka
#
#
##################

package Apache::admbookmarks;

use strict;
use Apache::Constants qw(:common);
use Apache::lonnet();

# --------------------------------------------------------------Put bookmarks

sub write_bookmarks {
    my $marks=shift;
    Apache::lonnet::put("nohist_bookmarks",('bookmarks' => $marks));
    return;
}

# --------------------------------------------------------------Get bookmarks

sub get_bookmarks {
    my %bookmark=Apache::lonnet::get('nohist_bookmarks',('bookmarks'));
    return %bookmark;
}

# ---------------------------------------------------Construct bookmark editor

sub construct_editor {
    my $bookmarks=shift;
    return(<<END_HTML) 
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>
<head>
<title>
Bookmark Tree Viewer/Editor
</title>
<script LANGUAGE="JavaScript">
var addressCache = "";
var dragCache = "";
var lastDrug = "";
var lastIcon = "";
var image_num=0;
var saveVar="";

function buildUserTree() {
this.dropCache = "";
this.dragCache = "";
this.bookmarks=new addFolder("Stuff",6,"bookmarks");
//-----------------------------------------------------------------------------
//                         The FOLLOWING data is supplied by the Perl Module
//-----------------------------------------------------------------------------
$bookmarks
//-----------------------------------------------------------------------------
//                         The PRECEDING data is supplied by the Perl Module
//-----------------------------------------------------------------------------
}
</script>
<script src="/res/adm/includes/bookmarklib.js" language="JavaScript"></script>
<script language="JavaScript">
       bookmarkpal = new buildUserTree();
</script>
</head>
<FRAMESET rows="25, *, 40, 25" topmargin=0 leftmargin=0 marginheight=0 marginwidth=0 frameborder="0" border="0" framespacing="0" onLoad="Javascript:initializeTree();">
 <FRAMESET cols="25, *, 25" frameborder="0" borders="0" framespacing="0">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_ul.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_uu.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_ur.html" marginwidth="0" marginheight="0" scrolling="no">
 </FRAMESET>
 <FRAMESET name="contentb" cols="25, *, 25" frameborder="0" borders="0" framespacing="0">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_left.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME name="content" src="/res/adm/pages/bookmarkmenu/loading_bookmarks.html" marginwidth="0" marginheight="0" scrolling="yes">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_right.html" marginwidth="0" marginheight="0" scrolling="no">
 </FRAMESET>
 <FRAMESET cols="25, *, 25" frameborder="0" borders="0" framespacing="0">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_left.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME name="toolbar" src="/res/adm/pages/bookmarkmenu/bookmarkmenu_toolbar.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_right.html" marginwidth="0" marginheight="0" scrolling="no">
 </FRAMESET>
 <FRAMESET cols="25, *, 25" frameborder="0" borders="0" framespacing="0">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_ll.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_bb.html" marginwidth="0" marginheight="0" scrolling="no">
  <FRAME src="/res/adm/pages/bookmarkmenu/bookmarkmenu_lr.html" marginwidth="0" marginheight="0" scrolling="no">
 </FRAMESET>
</FRAMESET>
</html>
END_HTML
}

# -------------------------------------------------------Construct Error window

sub construct_error {
  my $error_message=shift;
  return(<<END_ERROR)
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>
 <head>
  <title>
   Bookmark Tree Viewer/Editor  V3
  </title>
 </head>
 <body bgcolor="#BBBBBB">
  <table bgcolor="#FFFFFF" width="100%" height="90%" align="center">
   <td>
    <i>
     $error_message
    </i>
   </td>
  </table>
  <center>
   <form>
    <input type=button name="close" value="close window" onClick="javascript:window.close();">
   </form>
  </center>
 </body>
</html>
END_ERROR
}

# ---------------------------------------------Construct toolbar (after saving)

sub construct_toolbar {
  return(<<END_TOOLBAR)
<html>
<body>
<div align="bottom">
<center>
Bookmarks saved. 
[ <a href="/res/adm/pages/bookmarkmenu/bookmarkmenu_toolbar.html" target="toolbar">Continue</a> ]
</center>
</div>
</body>
</html>
END_TOOLBAR
}

# -----------------------------------------------------------------Main Handler

sub handler {
  my $r=shift;
  my %marks;
  my $save_mark;
  my $page;
  $r->content_type('text/html');
  $r->send_http_header;
  if (exists($ENV{'form.hiddenbookmarks'})) {
      $marks{'bookmarks'}=$ENV{'form.hiddenbookmarks'};
      write_bookmarks($marks{'bookmarks'});
      if ($marks{'bookmarks'} eq 'recover') {    #if statement is all debug stuff
	  my $recover=defaultmarks();
	  $r->print("BOOKMARK RECOVERY");
	  write_bookmarks($recover);
      }
      $page=construct_toolbar();
  } else {
      %marks=get_bookmarks();
      if (exists($marks{"con_lost"})) {
	  $page=construct_error("Connection broken with home server. Please contact your system administrator");
      } else {
	  if ( ($marks{'bookmarks'} eq "") or ($marks{'bookmarks'} eq "error:No such file or directory\n") ) {
	      $marks{'bookmarks'}=defaultmarks();
	  }
	  $page = construct_editor($marks{'bookmarks'});
      }
  }
  $r->print($page);
  return OK;
}

####################################
#
#          Here is a listing of default bookmarks. They have replaced 'recover'
#          These will load after all bookmarks have been deleted.
#                ben 10/12/2000
sub defaultmarks {
    return(<<END_MARKS);
this.bookmarks.p[0]=new addFolder("Default Bookmarks",6,"bookmarks.p[0]");
this.bookmarks.p[0].p[0]=new addFolder("Michigan State University",6,"bookmarks.p[0].p[0]");
this.bookmarks.p[0].p[0].p[0]= new addLink("Main Website","http://www.msu.edu","bookmarks.p[0].p[0]",0);
this.bookmarks.p[0].p[0].p[1]= new addLink("College of Natural Science","http://www.ns.msu.edu","bookmarks.p[0].p[0]",1);
this.bookmarks.p[0].p[0].p[2]= new addLink("College of Engineering","http://www.egr.msu.edu","bookmarks.p[0].p[0]",2);
this.bookmarks.p[0].p[0].p[3]= new addLink("Dept. of Mathematics","http://www.math.msu.edu","bookmarks.p[0].p[0]",3);
this.bookmarks.p[0].p[0].p[4]= new addLink("Lite Lab","http://www.lite.msu.edu","bookmarks.p[0].p[0]",4);
this.bookmarks.p[0].p[0].p[5]= new addLink("Physics and Astronomy","http://www.pa.msu.edu","bookmarks.p[0].p[0]",5);
this.bookmarks.p[0].p[1]= new addFolder("Lon-Capa",6,"bookmarks.p[0].p[1]");
this.bookmarks.p[0].p[1].p[0]= new addLink("Lon-Capa org","http://lon-capa.org","bookmarks.p[0].p[1]",0);
this.bookmarks.p[0].p[1].p[1]= new addLink("Lite Lab","http://www.lite.msu.edu","bookmarks.p[0].p[1]",1);
this.bookmarks.p[0].p[2]= new addLink("Redhat","http://redhat.com","bookmarks.p[0]",2);
this.bookmarks.p[0].p[3]= new addLink("Debian","http://debian.org","bookmarks.p[0]",3);
this.bookmarks.p[1]= new addLink("Other Bookmarks","http://kirk.lite.msu.edu","bookmarks",1);
END_MARKS
}

1;
__END__





