# The LearningOnline Network with CAPA
# Handler to set parameters for assessments
#
# (Handler to resolve ambiguous file locations
#
# (TeX Content Handler
#
# 05/29/00,05/30,10/11 Gerd Kortemeyer)
#
# 10/11,10/12,10/16 Gerd Kortemeyer)
#
# 11/20,11/21,11/22,11/23,11/24,11/25,11/27,11/28,12/08 Gerd Kortemeyer

package Apache::lonparmset;

use strict;
use Apache::lonnet;
use Apache::Constants qw(:common :http REDIRECT);
use GDBM_File;


my %courseopt;
my %useropt;
my %bighash;
my %parmhash;

my @outpar;

my @ids;
my %symbp;
my %mapp;
my %typep;
my %keyp;
my %defp;

my %allkeys;

my $uname;
my $udom;
my $uhome;

my $csec;

my $fcat;

# -------------------------------------------- Figure out a cascading parameter

sub parmval {
    my ($what,$id,$def)=@_;
    my $result='';
    @outpar=();
# ----------------------------------------------------- Cascading lookup scheme

       my $symbparm=$symbp{$id}.'.'.$what;
       my $mapparm=$mapp{$id}.'___(all).'.$what;

       my $seclevel=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$what;
       my $seclevelr=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$symbparm;
       my $seclevelm=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$mapparm;

       my $courselevel=
            $ENV{'request.course.id'}.'.'.$what;
       my $courselevelr=
            $ENV{'request.course.id'}.'.'.$symbparm;
       my $courselevelm=
            $ENV{'request.course.id'}.'.'.$mapparm;

# -------------------------------------------------------- first, check default

       if ($def) { $outpar[11]=$def;
                   $result=11; }

# ----------------------------------------------------- second, check map parms

       my $thisparm=$parmhash{$symbparm};
       if ($thisparm) { $outpar[10]=$thisparm;  
                        $result=10; }

# --------------------------------------------------------- third, check course

       if ($courseopt{$courselevel}) { $outpar[9]=$courseopt{$courselevel};  
                                       $result=9; }

       if ($courseopt{$courselevelm}) { $outpar[8]=$courseopt{$courselevelm}; 
                                        $result=8; }

       if ($courseopt{$courselevelr}) { $outpar[7]=$courseopt{$courselevelr}; 
                                        $result=7; }

       if ($csec) {

        if ($courseopt{$seclevel}) { $outpar[6]=$courseopt{$seclevel};  
                                    $result=6; }

        if ($courseopt{$seclevelm}) { $outpar[5]=$courseopt{$seclevelm};  
                                     $result=5; }  
 
        if ($courseopt{$seclevelr}) { $outpar[4]=$courseopt{$seclevelr};  
                                     $result=4; }
  
      }

# ---------------------------------------------------------- fourth, check user
      
      if ($uname) { 

       if ($useropt{$courselevel}) { $outpar[3]=$useropt{$courselevel};  
                                     $result=3; }

       if ($useropt{$courselevelm}) { $outpar[2]=$useropt{$courselevelm}; 
                                      $result=2; }

       if ($useropt{$courselevelr}) { $outpar[1]=$useropt{$courselevelr}; 
                                      $result=1; }

      }
     
    return $result;
}

# ------------------------------------------------------------ Output for value

sub valout {
    my ($value,$type)=@_;
    return
	($value?(($type=~/^date/)?localtime($value):$value):'&nbsp;&nbsp;');
}

# -------------------------------------------------------- Produces link anchor

sub plink {
    my ($type,$dis,$value,$marker,$return,$call)=@_;
    return '<a href="javascript:pjump('."'".$type."','".$dis."','".$value."','"
      .$marker."','".$return."','".$call."'".');">'.
      &valout($value,$type).'</a>';
}

# ================================================================ Main Handler

sub handler {
   my $r=shift;

   if ($r->header_only) {
      $r->content_type('text/html');
      $r->send_http_header;
      return OK;
   }

# ----------------------------------------------------- Needs to be in a course

   if (($ENV{'request.course.fn'}) && 
       (&Apache::lonnet::allowed('opa',$ENV{'request.course.id'}))) {
# -------------------------------------------------------- Variable declaration

      %courseopt=();
      %useropt=();
      %bighash=();

      @ids=();
      %symbp=();
      %typep=();

      my $message='';

      $csec=$ENV{'form.csec'};
      $udom=$ENV{'form.udom'};
      unless ($udom) { $udom=$r->dir_config('lonDefDomain'); }

      my $id=$ENV{'form.id'};
      if (($id) && ($udom)) {
          $uname=(&Apache::lonnet::idget($udom,$id))[1];
          if ($uname) {
	      $id='';
          } else {
              $message=
     "<font color=red>Unknown ID '$id' at domain '$udom'</font>";
          }
      } else {
          $uname=$ENV{'form.uname'};
      }
      unless ($udom) { $uname=''; }
      $uhome='';
      if ($uname) {
	  $uhome=&Apache::lonnet::homeserver($uname,$udom);
      
        if ($uhome eq 'no_host') { 
          $message=
     "<font color=red>Unknown user '$uname' at domain '$udom'</font>";
          $uname=''; 
        } else {
          $csec=&Apache::lonnet::usection(
				       $udom,$uname,$ENV{'request.course.id'});
          if ($csec eq '-1') {
             $message="<font color=red>".
              "User '$uname' at domain '$udom' not in this course</font>";
              $uname='';
              $csec=$ENV{'form.csec'};
	 } else {
              my %name=&Apache::lonnet::userenvironment($udom,$uname,
		('firstname','middlename','lastname','generation','id'));
              $message="\n<p>\nFull Name: ".
                          $name{'firstname'}.' '.$name{'middlename'}
	                 .$name{'lastname'}.' '.$name{'generation'}.
                       "<br>\nID: ".$name{'id'}.'<p>';
         }
        }
      }

      unless ($csec) { $csec=''; }

      $fcat=$ENV{'form.fcat'};
      unless ($fcat) { $fcat=''; }

# ------------------------------------------------------------------- Tie hashs
      if ((tie(%bighash,'GDBM_File',$ENV{'request.course.fn'}.'.db',
                       &GDBM_READER,0640)) &&
          (tie(%parmhash,'GDBM_File',
           $ENV{'request.course.fn'}.'_parms.db',&GDBM_READER,0640))) {

# --------------------------------------------------------- Get all assessments
	%allkeys=();
        %defp=();
        map {
	    if ($_=~/^src\_(\d+)\.(\d+)$/) {
	       my $mapid=$1;
               my $resid=$2;
               my $id=$mapid.'.'.$resid;
               my $srcf=$bighash{$_};
               if ($srcf=~/\.(problem|exam|quiz|assess|survey|form)$/) {
		   $ids[$#ids+1]=$id;
                   $typep{$id}=$1;
                   $keyp{$id}='';
                   map {
                       if ($_=~/^parameter\_(.*)/) {
			  my $key=$_;
                          my $allkey=$1;
                          $allkey=~s/\_/\./;
                          my $display=
			      &Apache::lonnet::metadata($srcf,$key.'.display');
                          unless ($display) {
                              $display=
			         &Apache::lonnet::metadata($srcf,$key.'.name');
                          }
                          $allkeys{$allkey}=$display;
                          if ($allkey eq $fcat) {
                             $defp{$id}=
                              &Apache::lonnet::metadata($srcf,$key.'.default');
			  }
                          if ($keyp{$id}) {
			      $keyp{$id}.=','.$key;
                          } else {
                              $keyp{$id}=$key;
		          }
		       }
                   } split(/\,/,
                      &Apache::lonnet::metadata($srcf,'keys'));
                   $mapp{$id}=
		       &Apache::lonnet::declutter($bighash{'map_id_'.$mapid});
                   $symbp{$id}=$mapp{$id}.
			'___'.$resid.'___'.
			    &Apache::lonnet::declutter($srcf);
	       }
            }
        } keys %bighash;
# ---------------------------------------------------------- Anything to store?
        if ($ENV{'form.pres_marker'}) {
       my ($sresid,$spnam,$snum)=split(/\&/,$ENV{'form.pres_marker'});
       $spnam=~s/\_/\./;
# ---------------------------------------------------------- Construct prefixes

       my $symbparm=$symbp{$sresid}.'.'.$spnam;
       my $mapparm=$mapp{$sresid}.'___(all).'.$spnam;

       my $seclevel=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$spnam;
       my $seclevelr=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$symbparm;
       my $seclevelm=
            $ENV{'request.course.id'}.'.['.
		$csec.'].'.$mapparm;

       my $courselevel=
            $ENV{'request.course.id'}.'.'.$spnam;
       my $courselevelr=
            $ENV{'request.course.id'}.'.'.$symbparm;
       my $courselevelm=
            $ENV{'request.course.id'}.'.'.$mapparm;

       my $storeunder='';
       if (($snum==9) || ($snum==3)) { $storeunder=$courselevel; }
       if (($snum==8) || ($snum==2)) { $storeunder=$courselevelm; }
       if (($snum==7) || ($snum==1)) { $storeunder=$courselevelr; }
       if ($snum==6) { $storeunder=$seclevel; }
       if ($snum==5) { $storeunder=$seclevelm; }
       if ($snum==4) { $storeunder=$seclevelr; }
       $storeunder=&Apache::lonnet::escape($storeunder);
 
       my $storecontent=
    $storeunder.'='.&Apache::lonnet::escape($ENV{'form.pres_value'}).'&'.
    $storeunder.'.type='.&Apache::lonnet::escape($ENV{'form.pres_type'});

       my $reply='';
           if ($snum>3) {
# ---------------------------------------------------------------- Store Course
            $reply=&Apache::lonnet::critical('put:'.
             $ENV{'course.'.$ENV{'request.course.id'}.'.domain'}.':'.
             $ENV{'course.'.$ENV{'request.course.id'}.'.num'}.':resourcedata:'.
             $storecontent,
             $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
           } else {
# ------------------------------------------------------------------ Store User
            $reply=
            &Apache::lonnet::critical('put:'.$udom.':'.$uname.':resourcedata:'.
             $storecontent,$uhome);
           }

         if ($reply=~/^error\:(.*)/) {
	     $message.="<font color=red>Write Error: $1</font>";
	 }
# ---------------------------------------------------------------- Done storing
   }
# -------------------------------------------------------------- Get coursedata
        my $reply=&Apache::lonnet::reply('dump:'.
              $ENV{'course.'.$ENV{'request.course.id'}.'.domain'}.':'.
              $ENV{'course.'.$ENV{'request.course.id'}.'.num'}.':resourcedata',
              $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
        if ($reply!~/^error\:/) {
           map {
             my ($name,$value)=split(/\=/,$_);
             $courseopt{&Apache::lonnet::unescape($name)}=
                        &Apache::lonnet::unescape($value);  
           } split(/\&/,$reply);
        }
# --------------------------------------------------- Get userdata (if present)
        if ($uname) {
           my $reply=
       &Apache::lonnet::reply('dump:'.$udom.':'.$uname.':resourcedata',$uhome);
           if ($reply!~/^error\:/) {
              map {
                my ($name,$value)=split(/\=/,$_);
                $useropt{&Apache::lonnet::unescape($name)}=
                         &Apache::lonnet::unescape($value);
              } split(/\&/,$reply);
           }
        }

# ------------------------------------------------------------------- Sort this

        @ids=sort  {  
           if ($fcat eq '') {
              $a<=>$b;
           } else {
              $outpar[&parmval($fcat,$a,$defp{$a})]<=>
              $outpar[&parmval($fcat,$b,$defp{$b})];
           } 
        } @ids;
# ------------------------------------------------------------------ Start page
         $r->content_type('text/html');
         $r->send_http_header;
	$r->print(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Assessment Parameters</title>
<script>

    function pclose() {
        parmwin=window.open("/adm/rat/empty.html","LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");
        parmwin.close();
    }

    function pjump(type,dis,value,marker,ret,call) {
        document.parmform.pres_marker.value='';
        parmwin=window.open("/adm/rat/parameter.html?type="+escape(type)
                 +"&value="+escape(value)+"&marker="+escape(marker)
                 +"&return="+escape(ret)
                 +"&call="+escape(call)+"&name="+escape(dis),"LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");

    }

    function psub() {
        pclose();
        if (document.parmform.pres_marker.value!='') {
            document.parmform.submit();
        } else {
            document.parmform.pres_value.value='';
            document.parmform.pres_marker.value='';
        }
    }

</script>
</head>
<body bgcolor="#FFFFFF" onUnload="pclose()">
<h1>Set Assessment Parameters</h1>
<form method="post" action="/adm/parmset" name="parmform">
<h2>Course: $ENV{'course.'.$ENV{'request.course.id'}.'.description'}</h2>
<b>
Section/Group: 
<input type="text" value="$csec" size="6" name="csec">
<br>
For User 
<input type="text" value="$uname" size="12" name="uname">
or ID
<input type="text" value="$id" size="12" name="id"> 
at Domain 
<input type="text" value="$udom" size="6" name="udom">
</b>
<input type="submit" value="Display">
<input type="hidden" value='' name="pres_value">
<input type="hidden" value='' name="pres_type">
<input type="hidden" value='' name="pres_marker">
ENDHEAD

        $r->print('<h2>'.$message.'</h2><p>Sort list by ');
	$r->print('<select name="fcat" onChange="this.form.submit();">');
        $r->print('<option value="">Enclosing Map</option>');
        map {
	    $r->print('<option value="'.$_.'"');
            if ($fcat eq $_) { $r->print(' selected'); }
            $r->print('>'.$allkeys{$_}.'</option>');
        } keys %allkeys;
        $r->print('</select>');
# ----------------------------------------------------------------- Start Table
        my $coursespan=$csec?8:5;
	 $r->print(<<ENDTABLEHEAD);
<p><table border=2>
<tr><td colspan=5></td>
<th colspan=$coursespan>Any User</th>
ENDTABLEHEAD
    if ($uname) {
	$r->print("<th colspan=3 rowspan=2>User $uname at Domain $udom</th>");
    }
    $r->print(<<ENDTABLETWO);
<th rowspan=3>Parameter in Effect</th></tr><tr><td colspan=5></td>
<th colspan=2>Resource Level</th>
<th colspan=3>in Course</th>
ENDTABLETWO
    if ($csec) {
	$r->print("<th colspan=3>in Section/Group $csec</th>");
    }
    $r->print(<<ENDTABLEHEADFOUR);
</tr><tr><th>Assessment URL and Title</th><th>Type</th>
<th>Enclosing Map</th><th>Part No.</th><th>Parameter Name</th>
<th>default</th><th>from Enclosing Map</th>
<th>general</th><th>for Enclosing Map</th><th>for Resource</th>
ENDTABLEHEADFOUR
    if ($csec) {
  $r->print('<th>general</th><th>for Enclosing Map</th><th>for Resource</th>');
    }
    if ($uname) {
  $r->print('<th>general</th><th>for Enclosing Map</th><th>for Resource</th>');
    }
	$r->print('</tr><tr>');
         my $defbgone='';
         my $defbgtwo='';
 	 map {
# ------------------------------------------------------ Entry for one resource
	     if ($defbgone eq '"E0E099"') {
		 $defbgone='"E0E0DD"';
             } else {
                 $defbgone='"E0E099"';
	     }
	     if ($defbgtwo eq '"FFFF99"') {
		 $defbgtwo='"FFFFDD"';
             } else {
                 $defbgtwo='"FFFF99"';
	     }
	    @outpar=();
            my $rid=$_;
            my $thistitle='';
            my %name=   ();
            my %part=   ();
	    my %display=();
	    my %type=   ();
            my %default=();
            my $uri=&Apache::lonnet::declutter($bighash{'src_'.$rid});

            map {
		$part{$_}=&Apache::lonnet::metadata($uri,$_.'.part');
                $name{$_}=&Apache::lonnet::metadata($uri,$_.'.name');
                $display{$_}=&Apache::lonnet::metadata($uri,$_.'.display');
                unless ($display{$_}) { $display{$_}=''; }
                $display{$_}.=' ('.$name{$_}.')';
                $default{$_}=&Apache::lonnet::metadata($uri,$_.'.default');
                $type{$_}=&Apache::lonnet::metadata($uri,$_.'.type');
                $thistitle=&Apache::lonnet::metadata($uri,$_.'.title');
            } split(/\,/,$keyp{$rid});

	    my $totalparms=scalar keys %name;
            $r->print('<td bgcolor='.$defbgone.
                ' rowspan='.$totalparms.'><tt><font size=-1>'.
                join(' / ',split(/\//,$uri)).
                '</font></tt><p><b>'.
                      $bighash{'title_'.$rid});
            if ($thistitle) {
		$r->print(' ('.$thistitle.')');
            }
            $r->print('</b></td>');
            $r->print('<td bgcolor='.$defbgtwo.
                    ' rowspan='.$totalparms.'>'.$typep{$rid}.'</td>');
            $r->print('<td bgcolor='.$defbgone.
                    ' rowspan='.$totalparms.'><tt><font size=-1>'.
		      join(' / ',split(/\//,$mapp{$rid})).'</font></tt></td>');
            map {
	       my $result=&parmval($part{$_}.'.'.$name{$_},$rid,$default{$_});

               $r->print("<td bgcolor=".$defbgtwo.
                  ">$part{$_}</td><td bgcolor=".$defbgone.
                  ">$display{$_}</td>");
               my $thismarker=$_;
               $thismarker=~s/^parameter\_//; 
               my $mprefix=$rid.'&'.$thismarker.'&';

               $r->print('<td bgcolor='.
                (($result==11)?'"#AAFFAA"':'#FFDDDD').'>'.
             &valout($outpar[11],$type{$_}).'</td>');
               $r->print('<td bgcolor='.
                (($result==10)?'"#AAFFAA"':'#FFDDDD').'>'.
             &valout($outpar[10],$type{$_}).'</td>');

               $r->print('<td bgcolor='.
                (($result==9)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[9],$mprefix.'9',
                    'parmform.pres','psub').'</td>');
               $r->print('<td bgcolor='.
                (($result==8)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[8],$mprefix.'8',
                    'parmform.pres','psub').'</td>');
               $r->print('<td bgcolor='.
                (($result==7)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[7],$mprefix.'7',
                    'parmform.pres','psub').'</td>');

               if ($csec) {
                 $r->print('<td bgcolor='.
                   (($result==6)?'"#AAFFAA"':$defbgtwo).'>'.
             &plink($type{$_},$display{$_},$outpar[6],$mprefix.'6',
                    'parmform.pres','psub').'</td>');
                 $r->print('<td bgcolor='.
                   (($result==5)?'"#AAFFAA"':$defbgtwo).'>'.
             &plink($type{$_},$display{$_},$outpar[5],$mprefix.'5',
                    'parmform.pres','psub').'</td>');
                 $r->print('<td bgcolor='.
                    (($result==4)?'"#AAFFAA"':$defbgtwo).'>'.
             &plink($type{$_},$display{$_},$outpar[4],$mprefix.'4',
                    'parmform.pres','psub').'</td>');
               }

               if ($uname) {
                 $r->print('<td bgcolor='.
                    (($result==3)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[3],$mprefix.'3',
                    'parmform.pres','psub').'</td>');
                 $r->print('<td bgcolor='.
                    (($result==2)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[2],$mprefix.'2',
                    'parmform.pres','psub').'</td>');
                 $r->print('<td bgcolor='.
                   (($result==1)?'"#AAFFAA"':$defbgone).'>'.
             &plink($type{$_},$display{$_},$outpar[1],$mprefix.'1',
                    'parmform.pres','psub').'</td>');
               }

               $r->print(
        '<td bgcolor=#CCCCFF>'.&valout($outpar[$result],$type{$_}).'</td>');
               $r->print("</tr>\n<tr>");
	   } sort keys %name;
# -------------------------------------------------- End entry for one resource
	 } @ids;
         $r->print('</table></form></body></html>');
         untie(%bighash);
	 untie(%parmhash);
      }
   } else {
# ----------------------------- Not in a course, or not allowed to modify parms
      $ENV{'user.error.msg'}=
        "/adm/parmset:opa:0:0:Cannot modify assessment parameters";
      return HTTP_NOT_ACCEPTABLE; 
   }
   return OK;
}

1;
__END__




